"use strict";

var sock = null;

var _logtimer = false;
var log = function(sz) {
	$(".status").stop().text(sz).show();
	
	if (_logtimer) {
		clearTimeout(_logtimer);
		_logtimer = false;
	};
	
	_logtimer = setTimeout(function() {
		$(".status").fadeOut('slow');
		_logtimer = false;
	}, 4000);
};

var _lastFrame = null;
var setFrame = function(fr) {
	$(".frame").removeClass("frame-active");
	$("#f_"+fr).addClass("frame-active");
	
	_lastFrame = fr;
};

var getFrame = function() {
	return _lastFrame;
};

//

var hesc = function(sz) {
	return $("<div>").text(sz).html();
};

//

var _hasSubmitted = false;

var _hand   = [];
var _submit = {};
var update_hand = function(hand) {
	_hand = hand;
	_submit = {};
	
	$("#lobby-hand a").remove();
	
	for (var i = 0, e = hand.length; i !== e; ++i) {
		$("#lobby-hand").append(
			$("<a>")
				.addClass('card')
				.attr('href', 'javascript:;')
				.text(cards.white[hand[i]])
				.attr('data-cardid', hand[i])
		);
	};
	
	if (_hasSubmitted) {		
		$("#lobby-submit")
			.attr('disabled', 'disabled')
			.attr('value', 'Submitted, waiting for round to end')
		;
		
	} else {

		$("#lobby-submit")
			.attr('disabled', 'disabled')
			.attr('value', 'Pick '+_question[0]+" card(s)")
		;

		$(".card").click(function() {
			var cardid = $(this).attr('data-cardid');
			if (cardid in _submit) {
				delete _submit[cardid];
				$(this).removeClass('card-selected');
			} else {
				_submit[cardid] = true;
				$(this).addClass('card-selected');
			}

			if (Object.keys(_submit).length === _question[0]) {
				$("#lobby-submit")
					.removeAttr('disabled')
					.attr('value', 'Submit')
				;
			} else {
				$("#lobby-submit")
					.attr('disabled', 'disabled')
					.attr('value', 'Pick '+_question[0]+" card(s)")
				;
			}
		});

	}
};

var _czar = '';
var _question = [];
var update_question = function(questionid) {
	_question = cards.black[questionid];
	$("#lobby-question").html(hesc(_question[1]) + " <small>"+hesc("(pick "+_question[0]+")")+"</small>");
	
	$("#czar-question").text(_question[1]);
};

var update_roomname = function(roomname) {
	$("#lobby-roomname").text(roomname);
};

var update_scores = function(scores) {
	var scoreinfo = '';
	for(var i in scores) {
		scoreinfo += i+": " + scores[i]+"\n";
	};
	$("#lobby-scores").text(scoreinfo);
};

var clear_answers = function() {
	$("#czar-options a").remove();
};

var _submissions = [];

var czar_refresh_button = function() {
	if (_mynick === _czar) {
		$("#czar-submit").show();
	} else {
		$("#czar-submit").hide();
	}
	
	if (_submissions.length === 0) {
		$("#czar-submit")
			.attr('disabled', 'disabled')
			.val('Waiting for submissions')
		;
	
	} else {
		if ($("#czar-options .card-selected").length === 1) {
			$("#czar-submit")
				.removeAttr('disabled', 'disabled')
				.val('Confirm')
			;
		} else {
			$("#czar-submit")
				.attr('disabled', 'disabled')
				.val('Pick winner')
			;
		}
		
	}
};

var _hassubmitted = false;
var update_answers = function(submissions) {
	_submissions  = submissions;
	_hassubmitted = false;
	
	clear_answers();
	
	for(var i = 0, e = submissions.length; i !== e; ++i) {
		var text = submissions[i].map(function(cid) { return cards.white[cid]; }).join("; ");
		
		$("#czar-options").append(
			$("<a>")
				.addClass('card')
				.attr('href', 'javascript:;')
				.attr('data-submissionid', i)
				.text(text)
		);
		
	};
	
	if (_mynick === _czar) {
		$("#czar-options .card").click(function() {
			if (! _hassubmitted) {
				$(this).toggleClass('card-selected');
				czar_refresh_button();
			}
		});
	}
	
	czar_refresh_button(); 
};

var set_has_submitted = function() {
	$("#czar-submit")
		.attr('disabled', 'disabled')
		.val('Submitted!')
	;
	
	_hassubmitted = true;
};

var _mynick = '';

//

var reset_game_state = function() {
	// clear globals
	
	update_hand([]);
	_question = [];
	
	// clear UI
	
	setFrame('ready');
	
	$("#ready-roomkey")
		.removeAttr('disabled')
		.val('')
	;
	
	$("#ready-join")
		.removeAttr('disabled')
		.attr('value', 'Join')
	;
	
	$("#ready-name").focus();
};

//

var roomlist_request = function() {
	sock.emit('listrooms');
	
	$("#ready-roomlist li").remove();
	
	$("#ready-refresh").attr('disabled', 'disabled').val('Refreshing...');
};

var roomlist_update = function(roomlist) {
	$("#ready-refresh").removeAttr('disabled').val('Refresh');
		
	$("#ready-roomlist li").remove();
	for (var i in roomlist) {
		$("#ready-roomlist").append(
			$("<li>").append(
				$("<a>")
					.attr('href', 'javascript:;')
					.attr('data-gamekey', roomlist[i]['key'])
					.text(roomlist[i]['name'] + ' - ' + roomlist[i]['playerct'] + ' player(s)')
			)
		);
	}

	$("#ready-roomlist a").click(function() {
		$("#ready-roomkey").val($(this).attr('data-gamekey'));
	});
};

//

var init_ws = function() {
	sock = io.connect("/"); //CONF.extern);

	sock.on('hi', function() {
		log('Connected to game server.');
		reset_game_state();
		roomlist_request();
	});
	
	sock.on('disconnect', function() {
		log('Lost connection to game server.');
	});
	
	sock.on('log', function(data) {
		log(data);
	});
	
	sock.on('crash', function(sz) {
		alert(sz);
		window.location.href = window.location.href; // refresh
	});
	
	sock.on('room_list', function(data) {
		roomlist_update(data);
	});
	
	sock.on('room_set', function(data) {
		if (getFrame() === "ready") {
			$("#ready-roomkey")
				.val(data)
				.removeAttr('disabled')
			;
			roomlist_request(); // update this too
		}
	});
	
	sock.on('room_info', function(data) {
		// name, players, czar, question
		
		_hasSubmitted = false;
		_czar = data.czar;
				
		if (_czar === _mynick) {
			clear_answers();
			
			log('You are the Card Czar for this round.');
			$("#czar-title").text('You are the Card Czar for this round.');
			setFrame('czar');
			
			czar_refresh_button();
			
		} else {
			log('Refreshed room. Czar is "'+data.czar+'"');
			setFrame('lobby');
			
			_hasSubmitted = false;
			update_hand(_hand);
		}
		
		update_roomname(data.name);

		update_question(data.question);

		update_scores(data.players);
		
		
	});
	
	sock.on('hand', function(data) {
		_hasSubmitted = false;
		update_hand(data);
	});
	
	sock.on('czar_select', function(data) {
		setFrame('czar');
		
		update_answers(data);
		
		if (_czar === _mynick) {
			$("#czar-title").text('Select a winner.');
		} else {
			$("#czar-title").text('Waiting for the Card Czar to select a winner.')
		}
	});
	
};

var init_handlers = function() {
	$("#ready-refresh").click(roomlist_request);
	
	$("#ready-newroom").click(function() {
		var gameName = prompt('Enter a name for the room:', "New Room");
		
		if (gameName) {
			$("#ready-roomkey")
				.attr('disabled', 'disabled')
				.attr('value', 'Creating new game...')
			;
			sock.emit('game_new', {name: gameName});
		}
	});
	
	$("#ready-join").click(function() {
		var myNick = $("#ready-name").val();
		var roomKey = $("#ready-roomkey").val();
		
		$("#ready-join")
			.attr('disabled', 'disabled')
			.attr('value', 'Joining...')
		;
		
		sock.emit('game_join', {
			'key': roomKey,
			'name': myNick
		});
		
		_mynick = myNick;
	});
	
	$("#lobby-submit").click(function() {
		sock.emit('submit', Object.keys(_submit));
		
		_hasSubmitted = true;
		
		// Remove selected cards from hand
		for (var i in _submit) {
			var idx = _hand.indexOf( i ); // find in hand
			_hand.splice(idx, 1); // remove
		};
		
		update_hand(_hand); // clears _submit
	});
	
	$("#czar-submit").click(function() {
		if (_mynick === _czar) {			
			var winner = _submissions[$("#czar-options .card-selected").attr('data-submissionid')];			
			sock.emit('czar_chooses', winner);
			
			set_has_submitted();
		}
	});
};

var onLoad = function() {
	init_ws();
	
	init_handlers();
};

window.addEventListener('load', onLoad);
